// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2012 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file Generic_commands.thrift
 *
 * CHANGELOG
 * 23 nov 2012  - Initiate header
 * 30 nov 2012  - Add cancel exception in get_key command
 *              - Change parameter of config_get_params from set type to list type
 * 10 dec 2012  - Change namespace to comply MPH coding rules.
 * 11 dec 2012  - Add terminal_echo command
 *              - Change return type of terminal_get_key command
 *              - Change service name to comply MPH coding rules.
 * 14 dec 2012  - Change Biofinger_enroll return type.
 *              - Change type names to comply MPH coding rules.
 * 18 dec 2012  - Add enable_intermediate_replies in enroll command.
 * 19 dec 2012  - Create terminal_get_version command.
 * 03 jan 2013  - Changed return type of user_DB_delete_users from void to
 *                map<string, bool> containing delete status of user list.
 *              - Added command "get_wiegand_or_clock_data_string".
 * 08 jan 2013  - Add types for terminal_get_status command
 * 09 jan 2013  - Update biofinger_authenticate_ref to specify the type of the reference templates.
 *              - user_DB_get_all_user_IDs is renamed to user_DB_get_user_IDs to be able to get all
 *                the user IDs of biometric database or all user IDs in white list
 * 14 jan 2013  - get_wiegand_or_clock_data_string -> wiegand_clock_data_get_string
 * 16 jan 2013  - Added command to get IP restraining list in form of string list.
 * 18 jan 2013  - Added command to delete IP from IP restraining list.
 * 18 jan 2013  - Added command wiegand_clock_data_send_string for sending wiegand or clock&data string.
 * 18 jan 2013  - Added commands of security password.
 * 19 jan 2013  - Added command "transaction_log_retrieve"
 *              - Added command "transaction_log_delete_all"
 *              - Added command "transaction_logs_retrieve_info"
 * 21 jan 2013  - Added command of reset parameters to factory settings.
 * 24 jan 2013  - updated argument type to binary in command"wiegand_clock_data_send_string"
 * 25 jan 2013  - Added command "transaction_log_get_status".
 *              - Renamed command "transaction_log_retrieve_info" to "transaction_log_get_fields".
 * 30 jan 2013  - Add enable_intermediate_replies parameter in Terminal_get_key command
 * 04 feb 2013  - Changed return type of get_sdac_status() function to Sdac_status structure.
 * 07 feb 2013  - Added command "retrieve_touchscreen_input".
 *              - Added get access schedule commands.
 *              - Added set access schedule commands.
 *              - Added reset access schedule commands.
 *              - Modified signatures of 'file_get', 'file_load', 'file_get_filenames', and 'file_erase' commands.
 * 12 feb 2013  - Added door open schedule commands
 *              - Modified access schedule commands and related exceptions.
 * 14 feb 2013  - Added Holiday schedule commands
 * 16 feb 2013  - Added "config_get_range" command.
 * 16 feb 2013  - Changed signature of 'file_get', and 'file_load'
 * 18 feb 2013  - Changed return type of "config_get_range" command.
 * 20 feb 2013  - Added  "configure_WIFI_network" and "scan_WIFI_networks" command.
 *              - Added "trigger_relay" command.
 *              - Modified command "reset_factory_settings".
 *              - Added "terminal_retrieve_peripherals" command.
 *              - Added "terminal_change_threat_level" command.
 * 21 feb 2013  - Added parameter in API get_terminal_configuration to retrieve interface name and IP version.
 * 21 feb 2013  - Modified parameter of function 'get_terminal_configuration'
 * 22 feb 2013  - Updated signature of 'file_get' to remove enabling of multiple chunks -- chunking is mandatory
 * 23 feb 2013  - Added "get_WIFI_network_status" command
 * 24 feb 2013  - Modified wifi related functions for throwing exception "WIFI_module_not_connected" exception
 * 05 mar 2013  - Added exceptions in commands: 'transaction_log_retrieve', 'terminal_set_configuration'
 *              - Removed intermediate reply required option from command 'transaction_log_retrieve'
 *              - Added "product_get_info" command.
 *              - Added "cls_write_admin_card" command.
 *              - Added "terminal_set_brightness" command.
 *              - Removed "modify_password" and "store_password" and added "password_set" command.
 *              - Renamed "verify_password" command to "password_verify" and modified it.
 *              - Renamed "delete_password" command to "password_reset" and modified it.
 *              - Removed "modify_passphrase", "store_passphrase" and "verify_passphrase" and added "passphrase_set" command.
 *              - Renamed "delete_passphrase" command to "passphrase_reset" and modified it.
 * 13 mar 2013  - Added commands for Event configuration settings. "set_events_configuration", "get_events_configuration" and "reset_events_configuration"
 * 13 mar 2013  - Exception added in "terminal_get_configuration" and "terminal_set_configuration"
 * 13 mar 2013  - "password_set" command's argument changed to 2 arguments. 1. password_old 2. password_new
 *              - removed command "set_cryptographic_key"
 *              - renamed command "modify_cryptographic_key" to "cryptographic_key_set"
 *              - renamed command "delete_cryptographic_key" to "cryptographic_key_reset"
 * 14 mar 2013  - "password_verify", "Security_default_password" exception added.
 *              - "passphrase_set", "Security_default_password" exception added.
 * 14 mar 2013  - Added 'NTP_server_config' & 'get_NTP_server_configuration' commands.
 * 15 mar 2013  - Add cls_authenticate_user
 *              - Change Terminal_settings_option parameter to Terminal_settings_type in terminal_get_configuration
 * 18 mar 2013  - Rename "set_events_configuration" and "get_events_configuration" into "events_set_config" and "events_get_config"
 *              - removed "reset_event_configuration"
 *              - Added licences (get,set) commands.
 *              - Added "cls_keys_load" and "cls_keys_reset" commands.
 *              - Removed "modify_cryptographic_key", "store_cryptographic_key", "verify_cryptographic_key" and "delete_cryptographic_key" commands.
 *              - Added Out_of_memory exception to Terminal_echo command
 *              - Added "cls_authenticate_user" command.
 *              - Change in terminal_get_configuration, Terminal_settings_type to Terminal_configuration_type
 * 25 mar 2013  - Added exception in command 'transaction_log_get_status'
 * 28 mar 2013  - Added commands 'config_set_video_phone_params' and 'config_get_video_phone_params'exception
 * 05 apr 2013  - Updated command 'user_DB_get_status', added argument user type
 * 10 apr 2013  - Added commands 'job_code_load_lists', 'job_code_retrieve_lists', 'job_code_check_value_against_user',
 *                'job_code_empty_lists' and 'job_code_remove_all_lists'
 * 15 apr 2013  - removed 'NTP_server_config' & 'get_NTP_server_configuration' commands.
 * 15 apr 2013  - Added command 'job_code_retrieve_list_indices'
 * 16 apr 2013  - Added exception in command 'job_code_retrieve_lists'
 * 06 may 2013  - Added exception in command wiegand_clock_data_get_string & wiegand_clock_data_send_string
 * 09 may 2013  - Updated command 'user_DB_get_users', added exception 'Generic_types.Invalid_argument_error'
 * 15 may 2013  - Changed 'enrol' word(UK English) to 'enroll' (US English change)
 * 21 may 2013  - Added "display_text" command.
 * 21 may 2013  - Added set/get/reset dynamic message data commands.
 * 22 may 2013  - Renamed "retrieve_touchscreen_input" to "retrieve_keypad_input"
 * 23 may 2013  - Removed command 'wiegand_clock_data_get_string'
 * 27 may 2013  - Added command 'send_to_host'
 *              - Added flag in 'user_db_set_users' to bypass similar finger check to improve the template transfer performance.
 * 17 jun 2013  - Modified argument of access_schedule_retrieve, access_schedule_delete, holiday_schedule_retrieve ,holiday_schedule_delete
 */

/** (continuing in another comments section as above comments section is too large for Thrift compiler)
 * 28 may 2013  - Renaming:
 *                         From                          To
 *                  get_ip_restrain_list        authorized_IP_get_list
 *                  add_ip_restrain             authorized_IP_add
 *                  delete_ip_restrain          authorized_IP_delete
 *                  get_ip_restrain_range_list  authorized_IP_get_range_list
 *                  add_ip_restrain_range       authorized_IP_add_range
 *                  delete_ip_restrain_range    authorized_IP_delete_range
 * 19 jun 2013  - Added command 'external_db_verify_id'.
 * 19 jun 2013  - Modified command name from 'external_db_verify_id' to 'external_db_verify_user'
 * 20 jun 2013  - Added exception 'Duplicated_finger_error' and 'DB_duplicate_record_error' in command 'user_DB_set_users'.
 * 24 jun 2013  - Modified command external_db_verify_user, added user id as input argument
 * 27 jun 2013  - Added exception 'Invalid_argument_error' in 'PIN_authenticate_db' command to return error if parameters are invalid.
 * 20 jul 2013  - Removed 'terminal_change_threat_level' distant command.
 *              - Modified 'licence' to 'license' in commands for spell correction.
 *              - Remove list of files from cls_write. Those files are now in card description
 * 25 jul 2013  - Updated command 'password_reset' and 'password_set'
 *              - Used enumeration for passphrase ID
 *              - Removed exception 'Security_default_password' from 'password_set' command
 *              - Added argument 'l1_keys' in command 'cls_keys_load'
 *              - Added exception 'License_not_found' in command 'config_set_params'
 *              - Added exception 'SD_card_not_found' in command 'licenses_add'
 *              - Removed "terminal_set_brightness" command.
 * 30 jul 2013  - Added exception 'License_not_found' in wifi related commands
 *              - Added exception 'Invalid_argument_error' in picture display command
 * 31 jul 2013  - Added exception Timeout_error in command retrieve_keypad_input to fix jira bug 606
 * 02 aug 2013  - Added exception 'SD_card_not_found' in command 'config_set_params'
 *              - Added exception 'SD_card_not_found' in dynamic messages related commands
 *              - Added exception '' for following commands:
 *                'file_load', 'file_get', 'file_erase'
 * 06 aug 2013  - Removed command 'terminal_get_status'
 * 07 aug 2013  - Polling command comment updated
 * 14 aug 2013  - Removed Firmware upgrade command
 * 16 aug 2013  - Updated command 'cls_keys_load', replaced all arguments by 'crypto_keys'
 *              - Renamed command 'cls_keys_load' to 'key_load'
 *              - Renamed command 'cls_keys_reset' to 'key_reset'
 * 27 aug 2013  - Added comments in command 'file_load'
 * 12 sep 2013  - Added comments in command 'retrieve_language_file_list'
 * 12 sep 2013  - Removed command 'terminal_get_key'
 * 18 oct 2013  - Added supported events configuration table in command 'events_set_config'
 * 26 oct 2013  - Thrift auto generation for csharp cannot parse 6000 character long line so broken comment in several lines for command 'events_set_config'
 * 26 oct 2013  - Added comment in command 'cls_write_user_card' that it supports only pkcompv2 template data only
 * 20 dec 2013  - Added command 'dynamic_message_get_user_IDs' to get list of User IDs having dynamic message
 * 26 dec 2013  - Added exception 'Generic_error' in 'throws' section of every command
 * 26 dec 2013  - Added command 'get_terminal_capabilities'
 * 07 feb 2014  - Modified comments in 'retrieve_language_file_list'
 * 20 feb 2014  - Add namespace for java and C# generation
 * 24 mar 2014  - Added product support notes
 * 05 jun 2014  - Added command 'switch_terminal_protocol'
 * 16 jun 2014  - Renamed command 'switch_terminal_protocol' to 'terminal_switch_protocol'
 * 30 jun 2014  - command MMI_set_configs/MMI_get_configs now internal command
 * 02 jul 2014  - command key_load, key_reset updated for USB script key
 * 03 jul 2014  - command terminal_switch_protocol moved under malite product macro for documentation generation
 * 11 Jul 2014  - command file_erase, file_get, file_get_filenames, file_load, get_terminal_capabilities, reset_factory_settings, terminal_get_configuration throws Generic_types.Invalid_argument_error
 * 20 aug 2014  - Added note in 'licenses_add' that reboot is required in case license is changed for user / transaction log database.
 * 25 aug 2014  - Added exception 'Invalid_argument_error' in command 'external_db_verify_user' for malite
 * 01 jan 2015  - Added exception 'SD_card_not_found' in following commands to notify user that SD card not supported so per user face rule can not be applied.
 *              - 'biofinger_enroll', 'user_DB_set_users' and 'external_db_verify_user'
 * 29 jan 2015  - MorphoWave Product support changes
 * 26 jun 2015  - Added new command to process HID elite card 'cls_HID_configure_card'
 * 07 jul 2015  - Rename 'cls_HID_configure_card' to 'cls_process_HID_configuration_card'
 * 20 jul 2015  - Rename 'cls_process_HID_configuration_card' to 'cls_process_special_card' and added Cancelled_error in this command
 * 05 aug 2015  - Added 'start_firmware_upgrade' command.
 * 01 oct 2015  - Added File_inexistent_error and Invalid_argument_error exception on 'start_firmware_upgrade' command.
 * 04 nov 2015  - Change java namespace for coherency
 *              - Rename CBI sensor to MorphoAccess SIGMA terminal
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 21 Nov 2016 - Remove 'capture_photo' from event table.
 * 04 Jan 2017 - Add exception 'Generic_types.Invalid_argument_error' for command 'retrieve_keypad_input'
 * 18 Jan 2017 - Added command "MMI_play".
 * 24 Mar 2017  - PRGCA Product support added
 * 22 June 2017 - Added exception 'Invalid_argument_error' in command 'key_load'
 * 24 July 2017 - Move 'Invalid_argument_error' in command 'key_load' exception at the end of the existing ones for backward compatibility.
 * 14 Sep 2017  - Updated information of "nb_of_finger" for peregrine product
 * 18 Oct 2017  - Added MMI_play support for MAExtreme
 * 20 Dec 2017  - Update description of 'key_load' command for OSDP secure channel base key
 * 07 Feb 2018  - Added 'transaction_log_delete' and 'transaction_log_tag_read' commands for transaction logs
 * 13 Feb 2018  - Update "biofinger_enroll" command description for peregrine product.
 * 15 Mar 2018  - Update command 'terminal_switch_protocol' description.
 * 12 Apr 2018  - Update information of exception  'DB_duplicate_record_error' in command 'biofinger_enroll'
 * 02 Jul 2018  - MA_VP_MD Product support added
 * 16 Nov 2018  - Added "MMI_play_advanced thrift" command
 * 16 Nov 2018  - Added "retrieve_keypad_input_advanced" command
 * 29 Nov 2018  - Added "distant_session_set_state_advanced" command
 * 03 dec 2018  - Added "display_text_advanced" command
 */
// </doc_history>

include "Common_generic_commands.thrift"
include "Db_types.thrift"
include "Generic_types.thrift"
include "Config_types.thrift"
include "Cls_types.thrift"
include "Biofinger_types.thrift"
include "File_types.thrift"
include "Security_types.thrift"

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd

service MA5G_generic_commands extends Common_generic_commands.Common_generic_commands
{
    /**************************************************
     * Job code commands
     **************************************************/

   /**
    * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
    * To load one or more job code lists
    *
    * This command will add job code list and if list is already
    * exist then it will update the list
    */
    void job_code_load_lists(   /** List number to be created */
                                1: list<Db_types.Jobcode_list> list_values)
        throws (    /** List data is not valid */
                    1: Db_types.Job_code_list_data_invalid jc_invalid_list_ex,
                    /** List array full, no more list can be added */
                    2: Db_types.Job_code_list_array_full jc_list_full_ex,
                    /** Invalid job code array size */
                    3: Db_types.Invalid_job_code_array_length jc_invalid_array_length_ex,
                    /** Generic exception */
                    4: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * To retrieve one or more job code lists data
     *
     * Return the job code list array.
     */
    list<Db_types.Jobcode_list> job_code_retrieve_lists(    /** List numbers to be retrieved */
                                                            1: set<i16> list_numbers)
               throws ( /** List number is not valid */
                        1: Db_types.Job_code_list_data_invalid jc_invalid_list_ex,
                        /** Generic exception */
                        2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * To retrieve all job code list number and name
     *
     * Return all job code list number and name.
     */
    list<Db_types.Jobcode_list> job_code_retrieve_list_indices()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Job code value validate against user
     */
    void job_code_check_value_against_user( /** User ID for which job code is to be checked */
                                            1: string user_id,
                                            /** Job code value to be validated */
                                            2: i32 jobcode_value)
        throws (    /** List number is not valid */
                    1: Db_types.Job_code_validation_failed jc_validation_failed_ex,
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * To remove one or more job code lists
     *
     * Return map of list number to the delete status, if successful then 'true'
     */
    map<i16, bool> job_code_empty_lists(    /** List array to be removed */
                                            1: set<i16> list_numbers)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * To remove all job code lists
     */
    void job_code_remove_all_lists()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**************************************************
     * Transaction log commands
     **************************************************/

    /**
     * Command to get the transaction log status.
     *
     * Return transaction log status values.
     */
    Db_types.Transaction_log_status transaction_log_get_status( /** Filter to be applied for getting transaction log count as per filter.*/
                                                                1: Db_types.Transaction_log_filter  filter)
        throws (    /** Invalid parameter set */
                    1: Generic_types.Invalid_argument_error invalid_arg_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Command to retrieve the transaction log entries.
     *
     * This command is multi response command. It will give data in multiresponse.
     *
     * Return list of the transaction log entries as per the filter applied.<br><br>
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * <pre>
     * void Generic_commandsClient::transaction_log_retrieve(std::vector< ::Distant_cmd::Transaction_log_DB_record> & _return, const  ::Distant_cmd::Transaction_log_filter& filter, const std::set< ::Distant_cmd::Transaction_log_DB_fields::type> & required_fields)
     * {
     *     send_transaction_log_retrieve(filter, required_fields);
     *     <span style="color:green">// recv_transaction_log_retrieve(_return); // Original line in the generate code</span>
     * &nbsp;
     *     <span style="color:green">// BEGIN: Custom section for receiving and processing intermediate data</span>
     *     uint32_t loop_id = 0;
     *     while (true)
     *     {
     *         recv_transaction_log_retrieve (_return);
     * &nbsp;
     *         <span style="color:green">// Check if the particular data chunk is final or not.
     *         //If it is the final data chunk, the data will be returned in normal way not as callback.</span>
     *         if (_return.empty() || _return[_return.size() - 1].is_final_response)
     *         {
     *             break;
     *         }
     *         else
     *         {
     *             <span style="color:green">// If the data chunk is not final, the data chunk will be returned as callback function manner
     *             // Following is the client's callback, if set, assumed to take necessary action on the received data,
     *             // e.g. Do process (store somewhere in file or memory) the retrieved transaction log chunk as other chunks are yet to be received.</span>
     *             if (Multiresponse_handler::client_tlog_command)
     *             {
     *                 Multiresponse_handler::client_tlog_command(_return);
     *             }
     *         }
     *     }
     *     <span style="color:green">// END: Custom section for receiving and processing intermediate data</span>
     * }
     * </pre>
     */
    list<Db_types.Transaction_log_DB_record> transaction_log_retrieve(  /** Filter to be applied for retrieving transaction logs.*/
                                                                        1: Db_types.Transaction_log_filter  filter,
                                                                        /** List of the transaction log fields to be retrieved */
                                                                        2: set<Db_types.Transaction_log_DB_fields>  required_fields)
        throws (    /** Invalid parameter set */
                    1: Generic_types.Invalid_argument_error invalid_arg_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Command to delete all the transaction log entries.
     */
    void transaction_log_delete_all()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Gets the available fields in the transaction log records
     *
     * Return the list of available transaction log record fields.
     */
    set<Db_types.Transaction_log_DB_fields> transaction_log_get_fields()
       throws ( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * Command to delete the transaction log(s).
     */
    void transaction_log_delete( /** Filter to be applied for removing transaction log(s).
                                 *  <code>tlog_stats</code> and <code>sort_field</code> are not applicable
                                 */
                                 1: Db_types.Transaction_log_filter  filter)
        throws (    /** Invalid parameter set */
                    1: Generic_types.Invalid_argument_error invalid_arg_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Command to apply read tag to the transaction log(s).
     */
    void transaction_log_tag_read( /** Filter to be applied for tagging transaction logs.
                                     *  <code>tlog_stats</code> and <code>sort_field</code> are not applicable
                                     */
                                     1: Db_types.Transaction_log_filter  filter)
        throws (    /** Invalid parameter set */
                    1: Generic_types.Invalid_argument_error invalid_arg_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**************************************************
     * Configuration commands
     **************************************************/

    /**
     * Returns a list of name of all terminal parameters
     */
     set<string> config_get_all_params_name()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Returns a list of values corresponding to requested parameters
     */
    list<Generic_types.Variant> config_get_params(  /** List of UTF8 string representing names of requested parameters */
                                                    1: list<string> param_list_UTF8)
        throws (    /** A requested parameter does not exist in terminal */
                    1: Config_types.Config_inexistent_parameter_error inexistant_param_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Set requested parameters with given values
     */
    void config_set_params( /**
                             * List of key-value pairs.<br>
                             * Parameters are represented by a UTF8 string
                             */
                            1: map<string, Generic_types.Variant> param_map)
        throws (    /** A requested parameter does not exist in terminal*/
                    1: Config_types.Config_inexistent_parameter_error   inexistant_param_xcept,
                    /** A parameter value is rejected by the terminal*/
                    2: Config_types.Config_invalid_value_error          invalid_value_xcept,
                    /** License not found exception */
                    3: Generic_types.License_not_found license_not_xcept,
                    /** SD card not found */
                    4:Generic_types.SD_card_not_found    sd_card_not_found_xcept,
                    /** Generic exception */
                    5: Generic_types.Generic_error generic_xcept);

    /**
     * Get the value range of the required or all configuration parameters.
     */
    map<string, Generic_types.Parameter_range> config_get_range( /**
                                                                  * List of key-value pairs.<br>
                                                                  * Parameters are represented by a UTF8 string
                                                                  */
                                                                  1: list<string> param_keys)
        throws (    /** A requested parameter does not exist in terminal */
                    1: Config_types.Config_inexistent_parameter_error inexistant_param_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**************************************************
     * Biometric commands
     **************************************************/
    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     * Use data contained in a contactless card to authenticate an user (biometric and/or PIN and/or BIOPIN check)<br><br>
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * Similar to the <a href=#SampleBiofingerIdentify>C++ sample code</a> provided for the method <code>biofinger_identify</code>
     */
     Cls_types.cls_authent_user_reply cls_authenticate_user(    /**
                                                                 * Max. duration of Smartcard detection process<br>
                                                                 * 0 implies an infinite process.
                                                                 */
                                                                1: i32  timeout_in_sec,
                                                                /** Contactless authentication parameters */
                                                                2: Cls_types.cls_authent_user_params authent_param)
        throws (    /** No finger or card was detected during process*/
                    1: Generic_types.Timeout_error              timeout_xcept,
                    /** Invalid parameter*/
                    2: Generic_types.Invalid_argument_error     invalid_argument_xcept,
                    /** Command is cancelled by distant command*/
                    3: Generic_types.Cancelled_error            cancelled_xcept,
                    /** Finger was misplaced or withdrawn during acquisition*/
                    4: Biofinger_types.Misplaced_finger_error   misplaced_finger_xcept,
                    /** Generic exception */
                    5: Generic_types.Generic_error              generic_xcept);

    /**
     * Identify an user using its finger against the terminal database
     *
     * The reference templates are stored in one (and only one) specified record.<br>
     * The search template is compared to all the templates found in the database
     *
     * <b>Warning:</b> If enable_intermediate_replies is true, you will need a modified version of the Thrift client that supports
     *          the reception of several T_REPLY messages for one command.<br><br>
     * <b id="SampleBiofingerIdentify">Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * <pre>
     * void Generic_commandsClient::biofinger_identify(::Distant_cmd::Biofinger_control_operation_reply& _return,
     *         const int8_t database_id, const int32_t timeout_in_sec, const int8_t threshold,
     *         const bool enable_intermediate_replies, const ::Distant_cmd::Biofinger_control_optional_param& optional_param)
     * {
     *     send_biofinger_identify(database_id, timeout_in_sec, threshold, enable_intermediate_replies, optional_param);
     *     <span style="color:green">// recv_biofinger_identify(_return); // Original line in the generated code</span>
     * &nbsp;
     *     <span style="color:green">// BEGIN: Custom section for receiving and processing intermediate data</span>
     *     uint32_t loop_id = 0;
     *     while (true)
     *     {
     *         <span style="color:green">// receive intermediate response</span>
     *         recv_biofinger_identify(_return);
     * &nbsp;
     *         <span style="color:green">// check for final_result status</span>
     *         if (_return.__isset.final_result)
     *         {
     *             break; <span style="color:green">// if final_result callback, break from loop</span>
     *         }
     *         else if (_return.__isset.cb_bio_command) <span style="color:green">// check for interested callback</span>
     *         {
     *             _return.__isset.cb_bio_command = false; <span style="color:green">// reset the callback flag</span>
     *             if (Multiresponse_handler::client_cb_bio_command)
     *             {
     *                 <span style="color:green">// process callback data</span>
     *                 Multiresponse_handler::client_cb_bio_command(_return.cb_bio_command);
     *             }
     *         }
     *         else if (_return.__isset.cb_low_resol_live_image)
     *         {
     *             <span style="color:green">// Process similar to "if (_return.__isset.cb_bio_command)" block</span>
     *         }
     *         else if (_return.__isset.cb_high_resol_capture_image)
     *         {
     *             <span style="color:green">// Process similar to "if (_return.__isset.cb_bio_command)" block</span>
     *         }
     *         else if (_return.__isset.cb_capture_quality)
     *         {
     *             <span style="color:green">// Process similar to "if (_return.__isset.cb_bio_command)" block</span>
     *         }
     *         else if (_return.__isset.cb_live_quality)
     *         {
     *             <span style="color:green">// Process similar to "if (_return.__isset.cb_bio_command)" block</span>
     *         }
     *     }
     *     <span style="color:green">// END: Custom section for receiving and processing intermediate data</span>
     * }
     * </pre>
     */
    Biofinger_types.Biofinger_control_operation_reply biofinger_identify(   /** Database identifier */
                                                                            1: byte     database_id,
                                                                            /**
                                                                             * Max. duration of identification process<br>
                                                                             * 0 implies an infinite process.
                                                                             */
                                                                            2: i32      timeout_in_sec,
                                                                            /**
                                                                             * This parameter specifies the value of the False Acceptance Ratio (FAR) of the MorphoSmart\99 device.<br>
                                                                             * The value of this parameter can be set from 0 to 10, by 1 value steps.
                                                                             */
                                                                            3: byte     threshold,
                                                                            /**
                                                                             * If set to false, you will receive only one reply containing the final result of the
                                                                             * identification, otherwise you may also receive asynchronous replies containing the
                                                                             * progress status of the identification.
                                                                             */
                                                                            4: bool enable_intermediate_replies,
                                                                            /** Identification optional parameters */
                                                                            5: Biofinger_types.Biofinger_control_optional_param optional_param)
        throws (    /** The user database is empty.*/
                    1: Db_types.DB_empty_error                  DB_empty_xcept,
                    /** No finger was detected during process*/
                    2: Generic_types.Timeout_error              timeout_xcept,
                    /** Invalid parameter*/
                    3: Generic_types.Invalid_argument_error     invalid_argument_xcept,
                    /** Command is cancelled by distant command*/
                    4: Generic_types.Cancelled_error            cancelled_xcept,
                    /** Finger was misplaced or withdrawn during acquisition*/
                    5: Biofinger_types.Misplaced_finger_error   misplaced_finger_xcept
                    /** Generic exception */
                    6: Generic_types.Generic_error              generic_xcept);

    /**
     * Authenticates an user using its finger against its database references
     *
     * The reference templates are stored in one (and only one) specified record.<br>
     * The search template is compared to all the templates found in the specified record
     * (the number of fingerprint templates depends on the database format).
     *
     * <b>Warning:</b> If enable_intermediate_replies is true, you will need a modified version of the Thrift client that supports
     *          the reception of several T_REPLY messages for one command.<br><br>
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * Similar to the <a href=#SampleBiofingerIdentify>C++ sample code</a> provided for the method <code>biofinger_identify</code>
     */
    Biofinger_types.Biofinger_control_operation_reply biofinger_authenticate_db(    /** Database identifier */
                                                                                    1: byte     database_id,
                                                                                    /**
                                                                                     * Max. duration of authentication process<br>
                                                                                     * 0 implies an infinite process.
                                                                                     */
                                                                                    2: i32      timeout_in_sec,
                                                                                    /**
                                                                                     * This parameter specifies the value of the False Acceptance Ratio (FAR) of the MorphoSmart\99 device.<br>
                                                                                     * The value of this parameter can be set from 0 to 10, by 1 value steps.
                                                                                     */
                                                                                    3: byte     threshold,
                                                                                    /** User ID to authenticate (UTF8 string)*/
                                                                                    4: string user_id_UTF8,
                                                                                    /**
                                                                                     * If set to false, you will receive only one reply containing the final result of the
                                                                                     * authentication, otherwise you may also receive asynchronous replies containing the
                                                                                     * progress status of the authentication.
                                                                                     */
                                                                                    5: bool enable_intermediate_replies
                                                                                    /** Authentication optional parameters */
                                                                                    6: Biofinger_types.Biofinger_control_optional_param optional_param)
        throws (    /** The specified user ID has not been found in the database.*/
                    1: Generic_types.Inexistent_user_id_error   inexistant_user_xcept
                    /** The user database is empty.*/
                    2: Db_types.DB_empty_error                  DB_empty_xcept,
                    /** No finger was detected during process*/
                    3: Generic_types.Timeout_error              timeout_xcept,
                    /** Invalid parameter*/
                    4: Generic_types.Invalid_argument_error     invalid_argument_xcept,
                    /** Command is cancelled by distant command*/
                    5: Generic_types.Cancelled_error            cancelled_xcept,
                    /** Finger was misplaced or withdrawn during acquisition*/
                    6: Biofinger_types.Misplaced_finger_error   misplaced_finger_xcept,
                    /** Generic exception */
                    7: Generic_types.Generic_error              generic_xcept);

    /**
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * Similar to the <a href=#SampleBiofingerIdentify>C++ sample code</a> provided for the method <code>biofinger_identify</code>
     */
    Biofinger_types.Biofinger_control_operation_reply biofinger_authenticate_ref(   /**
                                                                                     * Max. duration of authentication process<br>
                                                                                     * 0 implies an infinite process.
                                                                                     */
                                                                                    1: i32          timeout_in_sec,
                                                                                    /**
                                                                                     * This parameter specifies the value of the False Acceptance Ratio (FAR) of the MorphoSmart\99 device.<br>
                                                                                     * The value of this parameter can be set from 0 to 10, by 1 value steps.
                                                                                     */
                                                                                    2: byte         threshold,
                                                                                    /**
                                                                                     * List of reference templates<br>
                                                                                     * The templates can be in any format handled by the terminal<br>
                                                                                     * Max number of templates per list is 20
                                                                                     */
                                                                                    3: list<Biofinger_types.User_templates> ref_template_list,
                                                                                    /**
                                                                                     * If set to false, you will receive only one reply containing the final result of the
                                                                                     * authentication, otherwise you may also receive asynchronous replies containing the
                                                                                     * progress status of the authentication.
                                                                                     */
                                                                                    4: bool enable_intermediate_replies
                                                                                    /** Authentication optional parameters */
                                                                                    5: Biofinger_types.Biofinger_control_optional_param optional_param)
        throws (    /** A reference template is not supported by the terminal*/
                    1: Biofinger_types.Unsupported_format_error unsupported_format_xcept
                    /** A reference template does not match sensor security policy */
                    2: Biofinger_types.Incompatible_ref_error   incomptible_ref_xcept,
                    /** No finger was detected during process*/
                    3: Generic_types.Timeout_error              timeout_xcept,
                    /** Invalid parameter*/
                    4: Generic_types.Invalid_argument_error     invalid_argument_xcept,
                    /** Command is cancelled by distant command*/
                    5: Generic_types.Cancelled_error            cancelled_xcept,
                    /** Finger was misplaced or withdrawn during acquisition*/
                    6: Biofinger_types.Misplaced_finger_error   misplaced_finger_xcept,
                    /** Generic exception */
                    7: Generic_types.Generic_error              generic_xcept);

    /**
     * <b>MA5G:</b> Captures user fingers, optionally stores them in terminal database,
     * and/or returns finger templates, and/or returns fingerprint images.
     *
     * The template is calculated after three finger acquisitions (the user has to put each finger three times on the sensor).<br>
     * To obtain the best accuracy, users are advised to use the fore, the thumb or the middle fingers.<br>
     *
     * <b>MorphoWave Compact:</b> Capture 4 fingers in the hand swiped. The result of enrollment after minimum two consolidated swipes could be stored and/or returned.<br>
     *
     * <b>NOTE:</b> The <b>nb_of_finger</b> have to be set to 4 and for each finger, the finger position (Index, For finger...) and the expected status (Expected, Bandaged, Amputated)
     * have to be filled in <b>user_fields/User_templates</b>.<br>
     *
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * Similar to the <a href=#SampleBiofingerIdentify>C++ sample code</a> provided for the method <code>biofinger_identify</code>
     */
    Biofinger_types.Biofinger_enroll_operation_reply biofinger_enroll(  /**
                                                                         * Database identifier<br>
                                                                         * That parameter is useless if the function does not store in terminal database
                                                                         */
                                                                        1: byte                                             database_id,
                                                                        /**
                                                                         * Max. duration of authentication process<br>
                                                                         * 0 implies an infinite process. Allowed range is 0 - 65535 seconds
                                                                         */
                                                                        2: i32                                              timeout_in_sec,
                                                                        /**
                                                                         * This function can create a new record in terminal user database,
                                                                         * or can return the captured template, or can perform both.
                                                                         */
                                                                        3: Biofinger_types.Enrollment_type                  enrollment_type,
                                                                        /**
                                                                         * Number of fingers to enroll. <br>
                                                                         * It can be from 1 to 8 fingers for MorphoWave Tower terminal.<br>
                                                                         * It MUST be 4 for MorphoWave Compact terminal.<br>
                                                                         * For other products, it can be 1, 2 or 3 fingers.<br>
                                                                         * <note>In case of MorphoWave Tower and MorphoWave Compact terminals, each finger's position shall be set in user_fields.templates.template_number field of corresponding template.<br>
                                                                         * In case of MorphoWave Compact terminals, each finger's status shall be set in user_fields.templates.finger_status field of corresponding template.</note>
                                                                         */
                                                                        4: byte                                             nb_of_finger,
                                                                        /**
                                                                         * User ID to enroll<br>
                                                                         * That parameter is useless if the function does not store in terminal database
                                                                         */
                                                                        5: string                                           user_id_UTF8,
                                                                        /**
                                                                         * The user's data to store along with templates.<br>
                                                                         * <note>This parameter is useless if the function does not store user in terminal database.<br>
                                                                         * For MorphoWave Tower terminal, this parameter is used also to set fingers' position.<br>
                                                                         * For MorphoWave Compact terminal, this parameter is used also to set fingers' position and status.</note>
                                                                         */
                                                                        6: Db_types.User_DB_record                          user_fields,
                                                                        /**
                                                                         * If set to false, you will receive only one reply containing the final result of the
                                                                         * enrollment, otherwise you may also receive asynchronous replies containing the
                                                                         * progress status of the enrollment.
                                                                         */
                                                                        7: bool                                             enable_intermediate_replies,
                                                                        /** Enrollment optional parameter */
                                                                        8: Biofinger_types.Biofinger_enroll_optional_param  optional_param)
        throws  (   /** A captured template does not match sensor security policy */
                    1: Biofinger_types.Incompatible_ref_error   incompatbile_ref_xcept,
                    /** No finger was detected during process*/
                    2: Generic_types.Timeout_error              timeout_xcept,
                    /** Invalid parameter*/
                    3: Generic_types.Invalid_argument_error     invalid_argument_xcept,
                    /** Command is cancelled by distant command*/
                    4: Generic_types.Cancelled_error            cancelled_xcept,
                    /** User ID already exists in terminal database or another user is already enrolled using the same fingers*/
                    5: Db_types.DB_duplicate_record_error       duplicate_record_xcept,
                    /** Failure due to missing available space in the database*/
                    6: Db_types.DB_full_error                   DB_full_xcept,
                    /** The user record is containing a field which is not supported by the terminal database */
                    7: Db_types.User_DB_unavailable_field       unavailable_field_xcept,
                    /** User uses a finger more than once*/
                    8: Biofinger_types.Duplicated_finger_error  duplicated_finger_xcept,
                    /** Generic exception */
                    9: Generic_types.Generic_error              generic_xcept,
                    /** SD card not found */
                    10:Generic_types.SD_card_not_found          sd_card_not_found_xcept);

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Authenticates an user using its PIN code stored in terminal database
     *
     * User has to enter a PIN code on the terminal keypad.
     *
     * Return true, if the code keyed by the user matches the database user's PIN code, otherwise false.
     */
    bool PIN_authenticate_db(   /**
                                 * Max. duration of authentication process<br>
                                 * Limited to 60s.
                                 */
                                1: i32      timeout_in_sec,
                                /** User ID to authenticate (UTF8 string)*/
                                2: string   user_id_UTF8)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Reset selected parameters to factory settings.
     *
     */
    void reset_factory_settings(   /** List of the settings to reset.*/
                                    1: list<Generic_types.Terminal_reset_settings_type> list_of_settings)
        throws (
                /** Generic exception */
                1: Generic_types.Generic_error generic_xcept
                /** Invalid parameter*/
                2: Generic_types.Invalid_argument_error     invalid_argument_xcept);

     /**************************************************
     * User database commands
     **************************************************/

    /**
     * Gets the user's database status
     *
     * Return the database capacity and number of records.
     */
    Db_types.DB_status user_DB_get_status(  /** Type of user to retrieve status*/
                                            1: Db_types.User_type type)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Gets the available fields
     *
     * This command returns the list of available user fields in the terminal database.
     *
     * Return the list of available user fields.
     */
    set<Db_types.User_DB_fields> user_DB_get_fields()
       throws ( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);


    /**
     * Adds or modify one or more user records in the database
     *
     * Only the fields which are present in the DB_record structure will be modified.
     *
     * <b>Warning:</b> If the command is used to set non compressed templates, they all shall be of the same type.
     *  users list should be limited to 100 entries otherwise terminal may misbehave.
     */
    void user_DB_set_users( /** Mapping of User IDs (UTF8 string) to their database fields*/
                            1: map<string, Db_types.User_DB_record> users
                            /**
                              * If set to true,  checks  on  reference  templates  are performed:  same  finger  cannot  be  used  twice,<br>
                              * and  the  person  must  not  be  already  enrolled.<br>
                              * If set to false, these checks are not performed.<br>
                              * This  option  is  useful  to  reduce  the  time  taken  to  fill  large databases.<br>
                              * In this case, the database coherence must be previously checked.
                              */
                            2: bool   enable_similar_finger_check)
        throws (    /** Not all of the user records have been added or modified due to missing available space in the database*/
                    1: Db_types.DB_full_error                   DB_full_xcept,
                    /** One of the user record is containing a field which is not supported by the terminal database */
                    2: Db_types.User_DB_unavailable_field       unavailable_field_xcept,
                    3: Generic_types.Invalid_argument_error     invalid_argument_xcept,
                    /** Tried to enroll using duplicate fingers*/
                    4: Biofinger_types.Duplicated_finger_error  duplicate_finger_xcept,
                    /** Other user is already enrolled using the same fingers*/
                    5: Db_types.DB_duplicate_record_error       duplicate_user_xcept,
                    /** Generic exception */
                    6: Generic_types.Generic_error              generic_xcept,
                    /** SD card not found */
                    7:Generic_types.SD_card_not_found           sd_card_not_found_xcept);

    /**
     * Deletes one or more user record from the terminal database
     *
     * <b>Warning:</b> If a user is not found, no error is reported.
     */
    map<string, bool> user_DB_delete_records(   /** User IDs (UTF8 string) of records to delete*/
                                                1: set<string> user_IDs_UTF8)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);


    /**
     * Deletes all user records from the terminal database
     */
    void user_DB_delete_all_records()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Returns one or more user records.
     *
     * <b>Warning:</b> The list of user ID shall be limited to 100 items max.
     *
     * Return a map of UserIDs and their corresponding records.<br>
     * If a UserID is not found, it will not be present in the returned map.
     */
    map<string, Db_types.User_DB_record> user_DB_get_users( /** List of User IDs to return*/
                                                            1: set<string>                      user_IDs_UTF8,
                                                            /**
                                                             * Fields to return in the DB_record structure. It can be empty if you just
                                                             * want to check for a User ID presence.
                                                             */
                                                            2: set<Db_types.User_DB_fields> requested_fields)
       throws ( /** Invalid argument*/
                1: Generic_types.Invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * Returns the list of all user IDs (UTF8 string) present in terminal database based on user type
     *
     * Note: For the moment, only the users ID of users in biometric database or in white list can be retrieved.
     */
    set<string> user_DB_get_user_IDs( /** Kind of user to retrieve*/
                                      1: Db_types.User_type type)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**************************************************
     * Dynamic message commands
     **************************************************/

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Set dynamic message of users.
     *
     * <b>Warning:</b> Dynamic messages list should be limited to 500 entries otherwise terminal may misbehave.
     */
    void dynamic_message_set(   /** Mapping of User IDs (UTF8 string) to their dynamic message database fields*/
                                1: map<string, Db_types.Dynamic_msg_DB_record> dm_list)
        throws (    /** Invalid argument*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept,
                    /** SD card not found */
                    2:Generic_types.SD_card_not_found    sd_card_not_found_xcept,
                    /** Generic exception */
                    3: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Get dynamic message of users.
     *
     * <b>Warning:</b> User IDs list should be limited to 500 entries otherwise terminal may misbehave.
     *
     * Return the list of read files/data
     */
    map<string, Db_types.Dynamic_msg_DB_record> dynamic_message_get( /** Set of User IDs to return*/
                                                                     1: set<string> user_IDs_UTF8)
        throws (    /** Invalid argument*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept,
                    /** SD card not found */
                    2:Generic_types.SD_card_not_found    sd_card_not_found_xcept,
                    /** Generic exception */
                    3: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Reset dynamic message data of users.
     */
    void dynamic_message_reset( /** Set of User IDs whose dynamic message have to be reset*/
                                1: set<string> user_IDs_UTF8)
        throws (    /** Invalid argument*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept,
                    /** SD card not found */
                    2:Generic_types.SD_card_not_found    sd_card_not_found_xcept,
                    /** Generic exception */
                    3: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Returns the list of all user IDs (UTF8 string) present in terminal database having dynamic message
     */
    set<string> dynamic_message_get_user_IDs()
        throws (    /** SD card not found */
                    1:Generic_types.SD_card_not_found    sd_card_not_found_xcept,
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**************************************************
     * Polling commands
     **************************************************/

    /**
     * Get id from terminal polling buffer for external database verification.
     *
     * Return id for external database verification
     */
    string external_db_get_id()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Verify external database user data.
     */
    Generic_types.Verify_result_code external_db_verify_user(   /** User ID */
                                                                1: string     user_id_UTF8,
                                                                /** User data */
                                                                2: Db_types.User_DB_record user_data)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept,
                    /** SD card not found */
                    2:Generic_types.SD_card_not_found sd_card_not_found_xcept
                    /** Invalid argument*/
                    3: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**
     * Clears the id from terminal polling buffer
     */
     void external_db_clear_id()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**************************************************
     * Contactless commands
     **************************************************/

    /**
     * Get contactless card information<br>
     * Containing the Smartcard CSN as per the configuration, whether Standard CSN or Reverse CSN
     */
    Cls_types.Cls_info cls_get_info(    /** Duration max. of the operation in seconds*/
                                        1: i32 timeout_in_sec)
        throws (    /** No suitable cards were detected during the defined duration*/
                    1: Generic_types.Timeout_error ex1
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     * Write files on a contactless card.
     */
    void cls_write( /** Duration max. of the operation in seconds*/
                    1: i32                              timeout_in_sec,
                    /** List of card to support*/
                    2: Cls_types.Cls_cards_definition   cards)
        throws (    /** Timeout has been reached*/
                    1: Generic_types.Timeout_error          ex1,
                    /** Invalid argument*/
                    2: Generic_types.Invalid_argument_error ex2,
                    /** One of the specified contactless key is invalid*/
                    3: Cls_types.Cls_invalid_key_error      ex3,
                    /** Generic exception */
                    4: Generic_types.Generic_error          generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     * Encode an user contactless card
     *
     * The card is encoded according to terminal configuration.
     * The card will contains fields required for access on Morpho terminals.
     * This command supports only pkcompv2 template data write on card.
     */
    void cls_write_user_card(   /** Duration max. of the operation in seconds*/
                                1: i32                      timeout_in_sec,
                                2: Cls_types.Cls_user_card  card_data)
        throws (    /** Timeout has been reached*/
                    1: Generic_types.Timeout_error          ex1,
                    /** Invalid argument*/
                    2: Generic_types.Invalid_argument_error ex2,
                    /** Generic exception */
                    3: Generic_types.Generic_error          generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     * Encode an admin contactless card
     *
     * The card is encoded according to terminal configuration.
     * The card will contain the new contactless keys to load on other Morpho terminals.
     */
    void cls_write_admin_card(  /** Duration max. of the operation in seconds*/
                                1: i32                      timeout_in_sec,
                                /** Admin card type to encode*/
                                2: Cls_types.Cls_card_type  card_type)
        throws (    /** Timeout has been reached*/
                    1: Generic_types.Timeout_error              ex1,
                    /** Invalid argument*/
                    2: Generic_types.Invalid_argument_error     ex2,
                    /** The card detected by the terminal is not a card of input parameter card_type*/
                    3: Cls_types.Cls_invalid_card_type_error    ex3,
                    /** Generic exception */
                    4: Generic_types.Generic_error              generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     * Erase a contactless card
     *
     * For DESFire cards, according to the cards definition, this command will erase either one or several files,
     * either one or several applications or will format the card.<br>
     * In case of card formatting, the master PICC key will be set to 3DES legacy default value.
     *
     * For MIFARE cards, this command will set every writable sectors to 0xFF.<br>
     * Sector keys will be set to NXP default keys (0x00).
     */
    void cls_erase( /** Duration max. of the operation in seconds*/
                    1: i32                              timeout_in_sec,
                    /** List of card to support*/
                    2: Cls_types.Cls_cards_definition   cards)
        throws (    /** Timeout has been reached*/
                    1: Generic_types.Timeout_error          ex1,
                    /** Invalid argument*/
                    2: Generic_types.Invalid_argument_error ex2,
                    /** One of the specified contactless key is invalid*/
                    3: Cls_types.Cls_invalid_key_error      ex3,
                    /** Generic exception */
                    4: Generic_types.Generic_error          generic_xcept);

    /**
     * Load contactless keys<br><br>
     *
     * - For MIFARE cards, this command will load CRYPTO1 (MIFARE Classic) and/or AES (MIFARE Plus) keys.<br>
     * - For DESFire cards, this command will load 3DES and/or AES keys.<br>
     * - For iClass cards, this command will load the iClass key.<br>
     * - For RSA, this command will load the RSA key(s).<br>
     * - For l1 cards, this command will load the L1 site key.<br>
     * - For USB script, this command will load the USB script key.<br>
     * - For OSDP secure channel, this command will load the SCBK.<br/>
     */
    void key_load(  /** Crypto key object*/
                    1: Cls_types.Crypto_keys  crypto_keys)
        throws (    /** One of the specified contactless key is invalid*/
                    1: Cls_types.Cls_invalid_key_error  ex1,
                    /** Invalid argument*/
                    2: Generic_types.Generic_error      generic_xcept,
                    /** Generic exception */
                    3: Generic_types.Invalid_argument_error ex2);

    /**
     * Restore default contactless keys<br><br>
     *
     * - For MIFARE cards, this command will restore all MIFARE Classic and/or MIFARE Plus keys to default value.<br>
     *
     * - For DESFire cards, this command will restore all DESFire 3DES and/or DESFire AES keys to default value.<br>
     *
     * - For iClass cards, this command will restore the default iClass key.
     */
    void key_reset( /** List of keys to reset to default value*/
                    1: list<Cls_types.Cls_card_type>  types)
        throws (    /** Invalid argument*/
                    1: Generic_types.Invalid_argument_error ex1
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;N</product>
     * Read and process special types of card.<br>
     * As of now, command supports HID Keyroller card and HID configuration card.
     */
    void cls_process_special_card(  /** Duration max. of the operation in seconds*/
                                    1: i32 timeout_in_sec,
                                    /** HID configuration card type*/
                                    2: Cls_types.Cls_special_card_type card_type)
        throws (    /** Timeout has been reached*/
                    1: Generic_types.Timeout_error timeout_exception,
                    /** Invalid argument*/
                    2: Generic_types.Invalid_argument_error invalid_arg_exception,
                    /** The card detected by the terminal is not a card of input parameter card_type*/
                    3: Cls_types.Cls_invalid_card_type_error invalid_card_exception,
                    /** Generic exception */
                    4: Generic_types.Generic_error generic_xcept,
                    /** Command is cancelled by distant command*/
                    5: Generic_types.Cancelled_error cancelled_xcept);

    /**************************************************
     * Other commands
     **************************************************/

    /**
     * That functions is used to retrieve information like:
     * <ol>
     * <li> Terminal's local date and time </li>
     * <li> Terminal's IP configuration </li>
     * <li> Terminal's WIFI configuration </li>
     * <li> Terminal's GPRS configuration </li>
     * <li> Terminal's serial channel configuration </li>
     * </ol>
     */
    Generic_types.Terminal_configuration terminal_get_configuration(    /** Terminal setting options */
                                                                        1: Generic_types.Terminal_configuration_type terminal_settings_type)
        throws (    /** Failed to get Ethernet configuration */
                    1: Generic_types.Ethernet_config_not_found  ethernet_config_xcept
                    /** WIFI module not connected */
                    2: Generic_types.WIFI_module_not_connected  WIFI_mod_xcept,
                    /** License not found exception */
                    3: Generic_types.License_not_found          license_not_xcept
                    /** Generic exception */
                    4: Generic_types.Generic_error              generic_xcept
                    /** Invalid parameter*/
                    5: Generic_types.Invalid_argument_error     invalid_argument_xcept);

    /**
     * Get the list of terminal's predefined time zones
     *
     * The terminal contains a list of predefined time zones that can be used
     * to configure terminal's date and time (see terminal_set_configuration).
     *
     * Return a list of UTF-8 strings that represents predefined time zones handled
     * by the terminal
     */
    set<string> time_get_predefined_time_zone_list()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Open or close a distant session
     *
     * When a distant session is opened, the terminal only waits for distant commands.
     * No local operation can be done.
     */
    void distant_session_set_state( /** Open or Close the distant session */
                                    1: Generic_types.Distant_session_state state)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;Y;;Y;;Y</product>
     * Open or close a distant session <br>
     * Enable or disable the display message
     *
     * When display message is enabled with open session, the users are informed that the terminal is not "usable" for access.
     * <note> It is not recommended to use this command with "OFF" value for "display_message",<br>
     * without displaying any text/message, as it can give the impression to end user that the terminal<br>
     * is frozen, when he would interact with terminal during distant session </note>
     *
     * Return the distant session's states before execution and after execution of command.
     */
    Generic_types.Get_distant_session_state distant_session_set_state_advanced( /** Open or Close the distant session */
                                    1: Generic_types.Distant_session_state state,
                                    /** Optional parameter to Enable or disable the display message */
                                    2: Generic_types.Distant_session_optional_param optional_param)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);



    /**
     * Get the terminal's distant session state
     *
     * When a distant session is opened, the terminal only waits for distant commands.
     * No local operation can be done.
     *
     * Return the distant session's status (opened or closed)
     */
    Generic_types.Distant_session_state distant_session_get_state()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Cancel current delayed command
     */
    void cancel_operation()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     * Switch the terminal protocol to the specified one.
     *
     * Return the current terminal protocol (while this command has not changed the protocol).
     *
     * Note: The terminal protocol change shall come in effect only after the terminal reboot. Since firmware version 4.5 it is only possible to retrieve current protocol, changing protocol is no more available.
     */
    Generic_types.Terminal_protocol terminal_switch_protocol(   /** The terminal protocol to switch the terminal mode to */
                                                                1: Generic_types.Terminal_protocol protocol)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Get the specified capabilities of the terminal.
     */
    map< Generic_types.Capability, list< Generic_types.Variant > > get_terminal_capabilities (  /** List of the capabilities required */
                                                                                                1: set < Generic_types.Capability > capability_list)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error          generic_xcept
                    /** Invalid parameter*/
                    2: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**************************************************
     * Communication commands
     **************************************************/

    /**
     * Send wiegand or clock&data string
     */
    void wiegand_clock_data_send_string(    /** binary wiegand or clock & data data to send */
                                            1: binary   data_to_send)
        throws (    /** Incorrect Wiegand string as parameter */
                    1: Generic_types.Invalid_wiegand_string     invalid_wiegand_string_exception,
                    /** Wiegand output disabled on terminal */
                    2: Generic_types.Wiegand_output_disabled    wiegand_out_disable_exception,
                    /** Generic exception */
                    3: Generic_types.Generic_error              generic_xcept);

    /**************************************************
     * Multimedia commands
     **************************************************/

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Display a text message
     *
     * The terminal will display a text message on the screen.<br>
     * The text message will start at the specified coordinates and will be displayed
     * during the specified duration.<br>
     * Min-Max range for duration is (0-255 seconds) where 0 = infinite time.<br>
     * Min-Max range for X- co-ordinate and Y- co-ordinate is dependent on terminal LCD resolution and orientation.<br>
     */
    void display_text(  /** String message text to be display on terminal screen*/
                        1: string message_text,
                        /** Screen coordinates of the right-upper corner of the message text */
                        2: Generic_types.XY_coordinates coordinates,
                        /** Duration in seconds during which the text message is displayed */
                        3: i32 duration_in_sec)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error          generic_xcept);

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y</product>
     * Display a text message
     *
     * Terminal will display a text message on the screen.<br>
     * Text message will start at the specified coordinates and will be displayed
     * during the specified duration.<br>
     * Min-Max range for duration is (0-255 seconds) where 0 = infinite time.<br>
     * Min-Max range for X- co-ordinate and Y- co-ordinate is dependent on terminal LCD resolution and orientation.<br>
     * Color of background can be set with range (0x000000 - 0xFFFFFF)
     * Color of displayed text can be set with range (0x000000 - 0xFFFFFF)
     * Min-Max range for font size is dependent on terminal.
     */
    void display_text_advanced(  /** String message text to be display on terminal screen*/
                        1: string message_text,
                        /** Screen coordinates of the right-upper corner of the message text */
                        2: Generic_types.XY_coordinates coordinates,
                        /** Duration in seconds during which the text message is displayed */
                        3: i32 duration_in_sec
                        /** Setting of displayed text (background colour, text colour, font size) */
                        4: Generic_types.Display_text_optional_param display_text_optional_param)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error          generic_xcept);

    /**
     * Retrieve a file from the terminal.<br>
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * <pre>
     *  void Generic_commandsClient::file_get( ::Distant_cmd::File_chunk& _return, const  ::Distant_cmd::File_details& file_details)
     *  {
     *      send_file_get(file_details);
     *      <span style="color:green">// recv_file_get(_return); // Original line in the generated code</span>
     * &nbsp;
     *      <span style="color:green">// BEGIN: Custom section for receiving and processing intermediate data</span>
     *      while (true)
     *      {
     *          recv_file_get(_return); <span style="color:green">// Multiple reception in loop</span>
     * &nbsp;
     *          <span style="color:green">// Following is the client's callback assumed to take necessary action on the received data,
     *          //  e.g., keep writing the received partial data to the file until the last chunk of data is received.</span>
     *          Multiresponse_handler::client_cb_file_get(_return);
     * &nbsp;
     *          if(_return.is_last) <span style="color:green">// Exit the loop if the data received was last of all the multiple data received, i.e., no more data to be received</span>
     *          {
     *              break;
     *          }
     *      }
     *      <span style="color:green">// END: Custom section for receiving and processing intermediate data</span>
     *  }
     * </pre>
     */
    File_types.File_chunk file_get( /** File details of the file to obtain
                                     * (<code>File_types.File_details.name_UTF8</code> shall be ignored except where multiple files exist)
                                     */
                                    1: File_types.File_details file_details)
        throws( /** The specified file does not exist or insufficient data provided */
                1: Generic_types.File_inexistent_error  file_inexistent_xcept
                /** The SD card not found (might be required for specific files) */
                2: Generic_types.SD_card_not_found      SD_card_not_found_xcept
                /** Generic exception */
                3: Generic_types.Generic_error          generic_xcept
                /** Invalid parameter*/
                4: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**
     * Retrieve all the files name for a given file type
     *
     * Return list of structure for files' details
     */
    list<File_types.File_details> file_get_filenames( 1: File_types.File_type file_type)
        throws( /** Invalid file type specified */
                1: Generic_types.File_inexistent_error  file_inexistent_xcept
                /** Generic exception */
                2: Generic_types.Generic_error          generic_xcept);

    /**************************************************
     * SDAC commands
     **************************************************/

    /**
     * Return a structure containing 4 integers. The values in list are as following:
     * <ul>
     * <li> SDAC mode : 0=Off, 1=On </li>
     * <li> Relay state : 0=Door locked, 1=Door unlocked </li>
     * <li> Door status : 0=Closed, 1=Open, -1=Error </li>
     * <li> Request to exit state : 0=Off, 1=On, -1=Error </li>
     * </ul>
     */
    Generic_types.Sdac_status get_sdac_status()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Trigger the relay for a given duration.
     *
     * Note: A duration less or equal to 0 is forbidden.
     */
    void trigger_relay( /** duration in second for relay activation */
                        1: i32  duration_in_sec)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error          generic_xcept);

    /**************************************************
     * Security commands
     **************************************************/

    /**
     * Return a list of string of authorized IPs
     */
    list<string> authorized_IP_get_list(    /** IP protocol type i.e. v4, v6 or all */
                                            1: Generic_types.IP_protocol_type ip_protocol_type)
        throws( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * Add an authorized IP to communicate with
     */
    void authorized_IP_add( /** UTF8 string containing authorized IP */
                            1: string str_ip)
        throws( /** The database memory of authorized IP range list is full */
                1: Db_types.DB_full_error               DB_full_xcept,
                /** Invalid parameter*/
                2: Generic_types.Invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                3: Generic_types.Generic_error          generic_xcept);

    /**
     * Remove an authorized IP
     */
    void authorized_IP_delete(  /** UTF8 string containing authorized IP to be deleted */
                                1: string str_ip)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error     invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Return a list of string of authorized IP ranges
     */
    list<string> authorized_IP_get_range_list(  /** IP protocol type i.e. v4, v6 or all */
                                                1: Generic_types.IP_protocol_type ip_protocol_type)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Add an authorized IP range to communicate with
     */
    void authorized_IP_add_range(   /** UTF8 string containing authorized IP range start address */
                                    1: string str_start_ip,
                                    /** UTF8 string containing authorized IP range end address */
                                    2: string str_end_ip)
        throws( /** The database memory of IP authorized range list is full */
                1: Db_types.DB_full_error               DB_full_xcept,
                /** Invalid parameter*/
                2: Generic_types.Invalid_argument_error invalid_argument_xcept,
                /** Generic exception */
                3: Generic_types.Generic_error          generic_xcept);

    /**
     * Remove an authorized IP range
     */
    void authorized_IP_delete_range(    /** UTF8 string containing an authorized IP range start address to be deleted */
                                        1: string str_start_ip,
                                        /** UTF8 string containing an authorized IP range end address to be deleted */
                                        2: string str_end_ip)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error          generic_xcept);

    /**
     * To set a password
     *
     * Return true, if set successfully.
     */
    bool password_set(  /**
                         * password ID<br>
                         * Must be 0 (terminal password)
                         */
                        1: Security_types.Sec_obj_ID id,
                        /** old password */
                        2: string password_old,
                        /** new password */
                        3: string password_new)
        throws (    /** A default password detected */
                    1: Generic_types.Security_default_password  default_password_xcept,
                    /** Invalid parameter*/
                    2: Generic_types.Invalid_argument_error     invalid_argument_xcept,
                    /** Generic exception */
                    3: Generic_types.Generic_error              generic_xcept);

    /**
     * To verify a given password with stored password
     *
     * Return true, if verify successfully.
     */
    bool password_verify(   /**
                             * Password ID<br>
                             * Must be 0 (terminal password)
                             */
                            1: Security_types.Sec_obj_ID id,
                            /** Password to be verified with stored password*/
                            2: string password)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error     invalid_argument_xcept,
                    /** A default password detected */
                    2: Generic_types.Security_default_password  default_password_xcept,
                    /** Generic exception */
                    3: Generic_types.Generic_error              generic_xcept);

    /**
     * To reset a password
     *
     * Return true, if reset successfully.
     */
    bool password_reset(    /**
                             * Password ID<br>
                             * Must be 0 (terminal password)
                             */
                            1: Security_types.Sec_obj_ID id)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error          generic_xcept);

    /**
     * To set a passphrase
     *
     * Return true, if set successfully.
     */
    bool passphrase_set(    /**
                             * Passphrase ID
                             */
                            1: Security_types.Passphrase_id id,
                            /** Passphrase */
                            2: string passphrase)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error          generic_xcept);

    /**
     * To reset a passphrase
     *
     * Return true, if reset successfully.
     */
    bool passphrase_reset(  /**
                             * Passphrase ID
                             */
                            1: Security_types.Passphrase_id id)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error          generic_xcept);

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * To get user input from LCD GUI
     *
     * Return entered input from terminal
     */
    string retrieve_keypad_input(   /** Duration in second for getting input (valid range is from 0 to 2147483)*/
                                    1: i32  timeout_in_sec)
        throws (    /** Command is cancelled by user*/
                    1: Generic_types.Cancelled_error    cancelled_xcept,
                    /** User input form timed out*/
                    2: Generic_types.Timeout_error      timeout_xcept,
                    /** Generic exception */
                    3: Generic_types.Generic_error      generic_xcept,
                    /** Invalid parameter*/
                    4: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**
     * <product>Y;;N;;N;;N;;Y;;N</product>
     * To get user input from LCD GUI with custom message
     *
     * Return entered input on keypad from terminal
     */
    Generic_types.Keypad_input retrieve_keypad_input_advanced(   /** Keypad parameter configurations */
                                    1: Generic_types.Keypad_optional_param keypad_optional_param)
        throws (    /** Command is cancelled by user*/
                    1: Generic_types.Cancelled_error    cancelled_xcept,
                    /** User input form timed out*/
                    2: Generic_types.Timeout_error      timeout_xcept,
                    /** Generic exception */
                    3: Generic_types.Generic_error      generic_xcept,
                    /** Invalid parameter*/
                    4: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**************************************************
     * Access schedule commands
     **************************************************/

    /**
     * Command to get the access schedule entries.
     *
     * Return list of access_schedule data according to schedule_index.
     * List of Access_schedule provides data from Sunday to Saturday.
     */
    list<Db_types.Access_schedule>  access_schedule_retrieve(   /** Access schedule index.
                                                                 *  Value should be between 1 to 58.
                                                                 *   Schedule 0 is never allowed schedule.
                                                                 *   Schedule 63 is always allowed schedule.
                                                                 *   Schedule 59 to 62 are reserved schedule.
                                                                 */
                                                                1:list<byte> schedule_index)
        throws (    /** Invalid schedule index exception */
                    1: Db_types.Invalid_schedule_index                  invalid_schedule_index_ex,
                    /** Access schedule parameter not present in database */
                    2: Config_types.Config_inexistent_parameter_error   inexistant_param_xcept,
                    /** Generic exception */
                    3: Generic_types.Generic_error                      generic_xcept);

   /**
    * Command to set the access schedule entries.
    */
    void access_schedule_store( /** Access schedule data.
                                 *  List of Access_schedule takes data from Sunday to Saturday
                                 */
                                 1:list<Db_types.Access_schedule> schedule_data)
        throws (    /** Invalid schedule index exception */
                    1: Db_types.Invalid_schedule_index                  invalid_schedule_index_ex,
                    /** Access schedule parameter not present in database */
                    2: Config_types.Config_inexistent_parameter_error   inexistant_param_xcept
                    /** Access schedule data invalid */
                    3: Config_types.Config_invalid_value_error          invalid_value_xcept
                    /** Generic exception */
                    4: Generic_types.Generic_error                      generic_xcept);

    /**
     * Command to reset the access schedule entries.
     */
    void access_schedule_delete(    /** access schedule index. Value should be between 1 to 58.
                                     *   Schedule 0 is never allowed schedule.
                                     *   Schedule 63 is always allowed schedule.
                                     *   Schedule 59 to 62 are reserved schedule.
                                     */
                                     1:list<byte> schedule_index)
        throws (    /** Invalid schedule index exception */
                    1: Db_types.Invalid_schedule_index                  invalid_schedule_index_ex,
                    /** Access schedule parameter not present in database */
                    2: Config_types.Config_inexistent_parameter_error   inexistant_param_xcept
                    /** Access schedule data invalid */
                    3: Config_types.Config_invalid_value_error          invalid_value_xcept
                    /** Generic exception */
                    4: Generic_types.Generic_error                      generic_xcept);

    /**************************************************
     * Door open schedule commands
     **************************************************/

    /**
     * Command to get the door open schedule entries.
     *
     * Return list of Door_open_schedule data
     * List of Door_open_schedule provides data from Sunday to Saturday.
     */
    list<Db_types.Door_open_schedule>  door_open_schedule_retrieve()
        throws (    /** Door open schedule parameter not present in database */
                    1: Config_types.Config_inexistent_parameter_error inexistant_param_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Command to set the door open schedule entries.
     */
    void door_open_schedule_store(  /** Door open schedule data.
                                     *  List of Door_open_schedule takes data from Sunday to Saturday.
                                     */
                                     1:list<Db_types.Door_open_schedule> schedule_data)
        throws (    /** Door open schedule parameter not present in database */
                    1: Config_types.Config_inexistent_parameter_error   inexistant_param_xcept
                    /** Door open schedule data invalid */
                    2: Config_types.Config_invalid_value_error          invalid_value_xcept
                    /** Generic exception */
                    3: Generic_types.Generic_error                      generic_xcept);

    /**
     * Command to reset the door open schedule entries.
     */
    void door_open_schedule_delete()
        throws (    /** Door open schedule parameter not present in database */
                    1: Config_types.Config_inexistent_parameter_error   inexistant_param_xcept
                    /** Door open schedule data invalid */
                    2: Config_types.Config_invalid_value_error          invalid_value_xcept
                    /** Generic exception */
                    3: Generic_types.Generic_error                      generic_xcept);

    /**************************************************
     * Holiday schedule commands
     **************************************************/

    /**
     * Command to get the Holiday schedule entries.
     *
     * Return list of Holiday schedule data according to index
     *
     */
    list<Db_types.Holiday_schedule>  holiday_schedule_retrieve(     /** Holiday schedule index.
                                                                     *  Value should be between 0 to 63.
                                                                     */
                                                                    1:list<byte>  schedule_index)
        throws (    /** Invalid schedule index exception */
                    1: Db_types.Invalid_schedule_index                  invalid_schedule_index_ex,
                    /** Access schedule parameter not present in database */
                    2: Config_types.Config_inexistent_parameter_error   inexistant_param_xcept
                    /** Generic exception */
                    3: Generic_types.Generic_error                      generic_xcept);

    /**
     * Command to set the holiday schedule entries.
     */
    void holiday_schedule_store(    /** Holiday schedule data */
                                    1:list<Db_types.Holiday_schedule> schedule_data)
        throws (    /** Invalid schedule index exception */
                    1: Db_types.Invalid_schedule_index                  invalid_schedule_index_ex,
                    /** Access schedule parameter not present in database */
                    2: Config_types.Config_inexistent_parameter_error   inexistant_param_xcept
                    /** Access schedule data invalid */
                    3: Config_types.Config_invalid_value_error          invalid_value_xcept
                    /** Generic exception */
                    4: Generic_types.Generic_error                      generic_xcept);

    /**
     * Command to reset the holiday schedule entries.
     */
    void holiday_schedule_delete(   /** Holiday schedule index.
                                     *  Value should be between 0 to 63.
                                     */
                                     1:list<byte>  schedule_index)
        throws (    /** Invalid schedule index exception */
                    1: Db_types.Invalid_schedule_index                  invalid_schedule_index_ex,
                    /** Access schedule parameter not present in database */
                    2: Config_types.Config_inexistent_parameter_error   inexistant_param_xcept
                    /** Access schedule data invalid */
                    3: Config_types.Config_invalid_value_error          invalid_value_xcept
                    /** Generic exception */
                    4: Generic_types.Generic_error                      generic_xcept);

   /**
    * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
    * Command to scan WIFI networks
    * Return list of WIFI_network_info which contains network informations
    */
    list<Generic_types.WIFI_network_info> scan_WIFI_networks()
        throws (    /** WIFI module not connected */
                    1: Generic_types.WIFI_module_not_connected  WIFI_module_connect_xcept,
                    /** License not found exception */
                    2: Generic_types.License_not_found          license_not_xcept
                    /** Generic exception */
                    3: Generic_types.Generic_error              generic_xcept);

    /**
     Set one or more event configurations.<br />
     Following table shows which configurations are allowed for given event.<br /><br />
<div style="margin:0 0 0 30px">
	<table border="5" cellpadding="0" cellspacing="0" width="800"><colgroup><col width="185"/><col width="64" span="6"/></colgroup><tbody>
	<tr height="20"><td height="20" width="185"><strong>Event Name</strong></td><td width="64"><strong>Enable</strong></td><td width="64"><strong>Send To Cotroller</strong></td><td width="64"><strong>Trigger TTL0</strong></td><td width="64"><strong>Trigger TTL1</strong></td><td width="64"><strong>Trigger TTL2</strong></td><td width="64"><strong>Clock n Data ID</strong></td></tr>
	<tr height="20"><td height="20"><strong>tlac_duress_finger_detected</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td></tr>
	<tr height="20"><td height="20"><strong>tlac_fake_finger_detected</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td></tr>
	<tr height="40"><td height="40" width="185"><strong>tlac_user_control_successful</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td></td></tr>
	<tr height="20"><td height="20" width="185"><strong>tlac_biometric_mismatch</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td></tr>
	<tr height="20"><td height="20" width="185"><strong>tlac_pin_mismatch</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td></tr>
	<tr height="20"><td height="20" width="185"><strong>tlac_user_id_not_in_db</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td></tr>
	<tr height="20"><td height="20" width="185"><strong>tlac_control_timed_out</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td></tr>
	<tr height="20"><td height="20" width="185"><strong>tlac_rejected_by_schedule</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td> Yes </td></tr>
	<tr height="20"><td height="20" width="185"><strong>tlac_temp_validity_expired</strong></td><td> Yes</td><td> Yes </td><td></td><td></td><td></td><td> Yes </td></tr>
	<tr height="20"><td height="20" width="185"><strong>tlac_useruser_not_in_white_list</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td></tr>
	<tr height="20"><td height="20"><strong>tlac_black_listed_card</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td></tr>
	<tr height="20"><td height="20"><strong>tlac_face_not_detected</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td> Yes </td></tr>
	<tr height="20"><td height="20"><strong>tlac_multi_user_intermediate_id</strong></td><td> Yes </td><td></td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_transaction_log_full</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_controller_feedback_action</strong></td><td> Yes </td><td></td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_job_code_check_failure</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td> Yes </td></tr>
	<tr height="20"><td height="20"><strong>tlac_door_opened_for_too_long</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_door_forced_open</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_door_closed_after_alarm</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_door_unlocked</strong></td><td> Yes </td><td> Yes</td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_door_locked_back</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_management_menu_login</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_management_menu_logout</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_database_deleted</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_enrollment_completed</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_user_deleted</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_user_modification_completed</strong></td><td> Yes </td><td> Yes</td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_cls_card_encoded</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_cls_card_reset</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_settings_changed</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_cls_card_security_key_reset</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_security_policy_changed</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_tamper_detected</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td></tr>
	<tr height="20"><td height="20"><strong>tlac_tamper_cleared</strong></td><td> Yes </td><td></td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_terminal_boot_completed</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_firmware_upgrade</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_add_user</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_reboot_initiated</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_user_rule_check_failure</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td> Yes </td></tr>
	<tr height="20"><td height="20"><strong>tlac_triggers_blocked</strong></td><td> Yes </td><td></td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_triggers_back_to_normal</strong></td><td> Yes </td><td></td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_usr_timed_anti_passback</strong></td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td><td> Yes </td></tr>
	<tr height="20"><td height="20"><strong>tlac_sensor_recovery_start</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_sensor_recovery_end</strong></td><td> Yes </td><td> Yes </td><td></td><td></td><td></td><td></td></tr>
	<tr height="20"><td height="20"><strong>tlac_scratched_sensor</strong></td><td> Yes </td><td> Yes </td><td/><td/><td/><td/></tr>
	<tr height="20"><td height="20"><strong>tlac_dirty_sensor</strong></td><td> Yes </td><td> Yes </td><td/><td/><td/><td/></tr>
	<tr height="20"><td height="20"><strong>tlac_tamper_suspended</strong></td><td> Yes </td><td> Yes </td><td/><td/><td/><td/></tr>
	<tr height="20"><td height="20"><strong>tlac_tamper_back_to_monitor</strong></td><td> Yes </td><td> Yes </td><td/><td/><td/><td/></tr>
	</tbody></table></div>
     */
    void events_set_config( /** Mapping of Event IDs to their config structure fields*/
                            1: map<Db_types.Transaction_log_action_code, Generic_types.Event_config> events)
        throws (    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error          generic_xcept);

    /**
     * Retrieve one or more event configuration.
     *
     * Return a map of Event IDs and their corresponding records.<br>
     */
    map<Db_types.Transaction_log_action_code, Generic_types.Event_config> events_get_config(    /** List of Event IDs to return*/
                                                                                                1: set<Db_types.Transaction_log_action_code> event_IDs)
        throws (    /** Invalid parameter */
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Add license(s) to terminal
     * Note: User database and Transaction log database license change requires terminal reboot.
     */
    void licenses_add(  /** License file content */
                        1: binary license)
        throws (    /** Invalid parameter */
                    1:Generic_types.Invalid_argument_error  invalid_argument_xcept,
                    /** SD card not found */
                    2:Generic_types.SD_card_not_found       sd_card_not_found_xcept
                    /** Generic exception */
                    3: Generic_types.Generic_error          generic_xcept);

    /**
     * Retrieve the list of the licenses stored in the terminal
     * Return a list of the licenses contained in the terminal (empty list if the terminal doesn't contain license)
     */
    list<string> licenses_get()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Gets the list of video phone profiles.<br>
     * Note: Only Audio is supported in terminal where face camera is not available.
     */
    list<Generic_types.Video_phone_params> config_get_video_phone_params()
       throws ( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Sets the list of video phone profiles.<br>
     * Note: Only Audio is supported in terminal where face camera is not available.<br>
     * <B>Note:</B>
     * <ol>
     *  <li>A maximum of 20 profiles shall be stored.
     *  <li>If the number of profiles stored is less than 20, say 15, a fresh copy of the profiles,
     *          the 15 which are provided, and the remaining 5 blank profiles shall be stored.<br>
     *          For any GUI application, the blank profiles are like empty slots, which may not be required to be displayed.
     *  <li>To erase all profiles, the parameter <code>params_video_phone</code> with <i>zero</i> size must be passed.
     *  <li>To edit one or more profiles, and set them, the remaining profiles shall be preserved only if the latter profiles
     *          are again provided to set as is.<br>
     *          For instance, if you want to edit profiles #2 and #3, get all the profiles
     *          (via <code>config_get_video_phone_params</code>), edit the profiles of interest (profile #2 and #3),
     *          and using this API, set all the 20 profiles back. This way, the profiles other than profiles #2 and #3
     *          are preserved and not deleted. In case you pass <i>only</i> profiles #2 and #3 to this API, the rest of the profiles
     *          shall be stored as blank profiles, i.e., effectively deleting them.
     *  <li>Any profile entry with all of its parameters empty, i.e., empty strings and <code>0</code> for
     *          <code>Video_phone_params.port</code>, shall be considered as a blank profile (empty slot).
     *  <li>Any profile entry with not all of its parameters empty shall throw appropriate exception.
     * </ol>
     */
    void config_set_video_phone_params( /**
                                         * The list of video phone parameters. The list size to be maximum 20.
                                         */
                                         1: list<Generic_types.Video_phone_params> params_video_phone)
        throws (    /**
                     * Invalid IP string provided in <code>Video_phone_params.address_IP</code>.
                     */
                    1: Generic_types.Invalid_IP_error xcept_invalid_IP,
                    /**
                     * The number of profiles provided exceeded the maximum limit of 20.
                     */
                    2: Generic_types.Invalid_size_error xcept_invalid_size,
                    /**
                     * For any profile, incomplete information provided, i.e., both of the following conditions fail:
                     * <ul>
                     *   <li> All the parameters of the profile provided.
                     *   <li> None of the parameters of the profile provided (blank profile).
                     * </ul>
                     */
                    3: Config_types.Config_invalid_value_error xcept_invalid_value
                    /** Generic exception */
                    4: Generic_types.Generic_error generic_xcept)

    /**
     * That functions is used to send data on host:
     * <ol>
     * <li> Send data to host on IP network </li>
     * <li> Send data to host on serial channel </li>
     * </ol>
     */
    void send_to_host(  /**
                         * Structure that contains configuration of host channel. <br>
                         * It can contain several configurations.
                         */
                        1: Generic_types.Host_configuration host_config,
                        /** Data to send on host channel */
                        2: binary data)
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept,
                    /** Controller not found */
                    2: Generic_types.Host_not_found host_not_found_xcept
                    /** Generic exception */
                    3: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Retrieve the list of language files stored in terminal
     * Return a list of string of the language files
     */
    list<string> retrieve_language_file_list()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;N;;N;;Y;;N;;N;;N</product>
     * Start Firmware upgrade process
     */
    void start_firmware_upgrade(/**
                                 * Specify the type of upgrade<br>
                                 * As of now, only the MorphoWave Tower sensor is upgradeable.
                                 */
                                1: File_types.Upgrade_type type)
        throws (    /** Incompatible version*/
                    1: Generic_types.Biometric_sensor_incompatible_version_error incompatible_version_xcept,
                    /** The specified file does not exist or insufficient data provided */
                    2: Generic_types.File_inexistent_error file_inexistent_xcept,
                    /** Invalid parameter*/
                    3: Generic_types.Invalid_argument_error invalid_argument_xcept,
                    /** Generic exception */
                    4: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     * The command shall perform user control successful or fail
     * MMI operation (audio/buzzer/GUI message) on the terminal.
     */
     void MMI_play( /**
					  * specifies the MMI message configuration
					  */
					1: Generic_types.MMI_config mmi_config)
		throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error     invalid_argument_xcept,
					/** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;Y;;Y;;Y</product>
     * The command shall perform operations additional to MMI_play command (Enhanced MMI Play)
     * MMI operation (audio/buzzer/GUI message) Additional Operations (Sound Configuration/Relay Configuration/Display Configuration/Text Config)
     */
    void MMI_play_advanced ( /**
                              * specifies the parameters for Enhanced MMI play
                              */
        1: Generic_types.MMI_play_params mmi_play_params )
    throws (    /** Invalid parameter*/
        1: Generic_types.Invalid_argument_error     invalid_argument_xcept,
        /** Generic exception */
        2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;N;;N;;N;;N;;Y;;N</product>
     * Perform force enroll of user in current enrollment command.<br>
     * <note> - This operation is allowed only after indication by terminal in enrollment command that force finger can be performed.</note>
     */
     void force_enroll()
        throws (    /** Invalid parameter*/
                    1: Generic_types.Invalid_argument_error     invalid_argument_xcept,
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);
}
